from base64 import b64encode
from Crypto import Random
from Crypto.Cipher import DES3
from Crypto.Cipher import PKCS1_v1_5
from Crypto.Hash import MD5
from Crypto.PublicKey import RSA
from Crypto.Signature import PKCS1_v1_5 as PKCS1_v1_5_Signature

import random
import string
import requests
from requests.packages.urllib3.exceptions import InsecureRequestWarning
import json
import six
import sys

version = sys.version_info[0]
is_py2 = (version == 2)
is_py3 = (version == 3)

if is_py2:
    from urllib import quote_plus
    from urllib import unquote_plus
    from urllib import urlencode
elif is_py3:
    from urllib.parse import quote_plus
    from urllib.parse import unquote_plus
    from urllib.parse import urlencode
else:
    raise ImportError("urllib, quote_plus or unquote_plus cannot be imported!")

requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class Abenity(object):
    """
    Abenity API client
    """
    iv_size = 8  # initialization vector size
    des3_key_size = 24

    def __init__(self, username, password, api_key, version=2,
                 environment='live', timeout=10):
        """
        Constructor
        Args:
            username: API username or real username
            password: API token or user password
            api_key: API key
            version: API version
            environment: 'live' or 'sandbox'
            timeout: timeout in seconds
        """

        self._username = username
        self._password = password
        self._api_key = api_key
        self._version = int(version)
        self._environment = environment
        self._api_url = 'https://api.abenity.com'
        if environment == 'sandbox':
            self._api_url = 'https://sandbox.abenity.com'
        self._timeout = timeout

        self._public_key = 'ssh-rsa AAAAB3NzaC1yc2EAAAADAQABAAAAgQC8NVUZU' + \
                           'tr2IHiFoY8s/qFGmZOIewAvgS4FMXWZ81Qc8lkAlZr9e1' + \
                           '71xn4PgKr+S7YsfCt+1XKyo5XmrJyaNUe/aRptB93NFn6' + \
                           'RoFzExgfpkooxcHpWcPy+Hb5e0rwPDBA6zfyrYRj8uK/1' + \
                           'HleFEr4v8u/HbnJmiFoNJ2hfZXn6Qw== phpseclib-ge' + \
                           'nerated-key'

        self._triple_des_key = ''.join([
            random.choice(string.ascii_letters + string.digits)
            for n in range(self.des3_key_size)
            ])

    def set_triple_des_key(self, key=None):
        if key is None:
            self._triple_des_key = ''.join([
                random.choice(string.ascii_letters + string.digits)
                for n in range(self.des3_key_size)
                ])
        else:
            self._triple_des_key = key

    def _send_request(self, api_method, http_method='GET', data={}):
        params = {'api_username': self._username,
                  'api_password': self._password,
                  'api_key': self._api_key}
        params.update(data.items())

        api_url = self._api_url+'/v'+str(self._version)+'/client'+api_method
        headers = {'user-agent': 'abenity/abenity-python)'}
        response = {}

        if http_method == 'GET':
            response = requests.get(api_url, verify=False, headers=headers,
                                    params=params, timeout=self._timeout)
        elif http_method == 'POST':
            response = requests.post(api_url, verify=False, headers=headers,
                                     data=params, timeout=self._timeout)
        elif http_method == 'DELETE':
            response = requests.delete(api_url, verify=False, headers=headers,
                                     data=params, timeout=self._timeout)

        return json.loads(response.text)

    def _encrypt_payload(self, payload, iv):
        cipher = DES3.new(six.b(self._triple_des_key),
                          DES3.MODE_CBC,
                          IV=iv)

        # If the size of the data is not n * blocksize,
        # the data will be padded with '\0'.
        padding = '\0' * (self.iv_size - len(payload) % self.iv_size)
        payload_padded = payload + padding

        payload_encrypted = cipher.encrypt(six.b(payload_padded))
        payload_encrypted_base64 = b64encode(payload_encrypted).decode("utf-8")
        return quote_plus(payload_encrypted_base64) + "decode"

    def _generate_rsa_key(self, key):
        return RSA.importKey(key)

    def _encrypt_cipher(self, rand_function=None):
        key = self._generate_rsa_key(self._public_key)
        if rand_function is None:
            cipher = PKCS1_v1_5.new(key)
        else:
            cipher = PKCS1_v1_5.new(key, randfunc=rand_function)

        des3_key_bytes = six.b(self._triple_des_key)

        des3_key_encrypted = cipher.encrypt(des3_key_bytes)
        des3_key_enc_base64 = b64encode(des3_key_encrypted).decode('utf-8')
        des3_key_enc_base64_encoded = quote_plus(des3_key_enc_base64) + \
            "decode"
        return des3_key_enc_base64_encoded

    def _sign_message(self, payload_encrypted_base64_urlencoded, private_key):
        key = self._generate_rsa_key(private_key)
        signer = PKCS1_v1_5_Signature.new(key)
        payload = unquote_plus(payload_encrypted_base64_urlencoded[:-6])
        md5_hash = MD5.new(payload.encode("utf-8"))
        signature = signer.sign(md5_hash)
        signature_base64 = b64encode(signature).decode("utf-8")
        return quote_plus(signature_base64) + "decode"

    def sso_member(self, member_profile, private_key, init_vector=None):
        """
        Single Sign-On a member

        Args:
            member_profile: A dict of key/value pairs that describes the member
            private_key: Your RSA private key, used to sign your message

        Returns:
            The raw API response string
        """

        payload = urlencode(member_profile)

        # Create initialization vector (iv)
        if init_vector is None:
            init_vector = Random.get_random_bytes(self.iv_size)

        iv_urlencoded = quote_plus(b64encode(init_vector).decode("utf-8")) + \
            "decode"

        payload_encrypted = self._encrypt_payload(payload, init_vector)
        encrypted_inner_key = self._encrypt_cipher()
        signature = self._sign_message(payload_encrypted, private_key)

        request_data = {
                        'Payload': unquote_plus(payload_encrypted),
                        'Cipher': unquote_plus(encrypted_inner_key),
                        'Signature': unquote_plus(signature),
                        'Iv': unquote_plus(iv_urlencoded)
                        }
        return self._send_request('/sso_member.json', 'POST', request_data)



from __future__ import unicode_literals

from abenity.client import Abenity
from base64 import b64encode
from Crypto.Cipher import PKCS1_v1_5
from Crypto.PublicKey import RSA
import datetime
import pprint
import binascii
from random import randint
import pytest
import six
import sys

version = sys.version_info[0]
is_py2 = (version == 2)
is_py3 = (version == 3)

if is_py2:
    from urllib import quote_plus
    from urllib import unquote_plus
    from urllib import urlencode
elif is_py3:
    from urllib.parse import quote_plus
    from urllib.parse import unquote_plus
    from urllib.parse import urlencode
else:
    raise ImportError("urllib quote_plus or unquote_plus cannot be imported!")


class TestClass:
    abenity_api_username = 'username'
    abenity_api_password = 'password'
    abenity_api_key = 'EP89f4pVQbEoUXc3JcT7asdktThBVR3K'

    f = open('private-key-test.txt', 'r')
    abenity_your_private_key = f.read()

    member_profile = {'creation_time': datetime.datetime.utcnow().isoformat() + 'Z',
                      'salt': 10000,
                      'send_welcome_email': 1,
                      'client_user_id': '1',
                      'email': 'john@acme.com',
                      'firstname': 'John',
                      'lastname': 'Smith',
                      'address': '2134 Main Street',
                      'city': 'Mountain View',
                      'state': 'CA',
                      'zip': '90210',
                      'country': 'US'}

    abenity = Abenity(abenity_api_username, abenity_api_password,
                      abenity_api_key, version=2, environment='sandbox',
                      timeout=10)

    def test_set_triple_des_key(self):
        self.abenity.set_triple_des_key('IoAZqZ3RCezxj7eKWeKj00mT')
        assert self.abenity._triple_des_key == 'IoAZqZ3RCezxj7eKWeKj00mT'

    def test_encrypt_payload(self):
        self.abenity.set_triple_des_key('IoAZqZ3RCezxj7eKWeKj00mT')
        payload = urlencode(self.member_profile)
        init_vector = binascii.unhexlify("41420c2d32373c3e")
        payload_encrypted = self.abenity._encrypt_payload(payload, init_vector)
        # TODO: add a better test, right now random and no easy way to override
        #       an easy function to make it not random, that I can see.
        assert payload_encrypted is not None

    def test_encrypt_cipher(self):
        self.abenity.set_triple_des_key('IoAZqZ3RCezxj7eKWeKj00mT')

        # remove RSA randomness, so can test
        def newRand(obj=None):
            return six.b('a')

        cipher = self.abenity._encrypt_cipher(rand_function=newRand)
        assert cipher == 'vDUtFs8K8pvy0db7D1EXTrUkWv5PTeldb77BBXpQN7SM521b' + \
            'INUcgbDbNc5Cn66ukeSNykeYPSAA4fzX3CGb1fuuO5stE19FP8cabJZFQL2GL' + \
            '6UrWu%2FAz6tAx06hl4R9CUSLu0ZQPwweW%2BrLKeFluvnlHIMsdzdmiybeuS' + \
            'UhucE%3Ddecode'

    def test_sign_message(self, monkeypatch):
        self.abenity.set_triple_des_key('IoAZqZ3RCezxj7eKWeKj00mT')

        # remove RSA randomness, so can test
        def mockreturn(key=None):
            keyObj = RSA.importKey(key)

            def newRand(obj=None):
                return six.b('abc')
            keyObj._randfunc = newRand
            return keyObj

        monkeypatch.setattr(self.abenity, '_generate_rsa_key', mockreturn)
        dummy_encrypted_payload = "asdf09asdf8a0s8df0a8sd09f8a0s98df0as8df09ad"
        cipher = self.abenity._sign_message(dummy_encrypted_payload,
                                            self.abenity_your_private_key)
        assert cipher == 'LL6DS2gEQxwYUjxfNQZdKOtv1MSP7bpK0PP9LdQ%2FYG3pF' + \
            '1Sy2e2LYxkZt6M9HidkXMKc3F73gVsiBomj91YRAlKV23VNMY31wNeMmISVi' + \
            'jk24vmbNttI6Pqy6In0kOtqUgVGCJDWSMPC2P7RWL655J%2FE7QF7f0MizF5' + \
            '3Gk3DoSgx7PuNigz9YYPDe%2BSOcIpdAUOYaeJ8ahMje8cij0uOi0%2FORPr' + \
            'jvb9t4RSvO2%2FKilY%2BDn2QvMFH79l91BsZ8EpCNxgnhSAiYQXuCTnd32W' + \
            'fTWLfKvj4smxtk9TbiCIueiM9DmtIU1np8OVruu%2FFwNxnHwD09ssL8fYz1' + \
            'PLfkE%2BJhw%3D%3Ddecode'

    def test_request_data(self):
        self.abenity.set_triple_des_key('IoAZqZ3RCezxj7eKWeKj00mT')
        payload = urlencode(self.member_profile)
        init_vector = binascii.unhexlify("41420c2d32373c3e")
        iv_urlencoded = quote_plus(b64encode(init_vector).decode("utf-8")) + \
            "decode"



#!/usr/bin/env python

from setuptools import setup
from setuptools.command.test import test as TestCommand
import sys


class PyTest(TestCommand):
    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = ['--strict', '--verbose', '--tb=long', '-s', 'tests']
        self.test_suite = True

    def run_tests(self):
        import pytest
        errno = pytest.main(self.test_args)
        sys.exit(errno)


setup(
    name='abenity',
    packages=['abenity'],
    version='1.4.0',

    description='Abenity API client',
    long_description='A Python library for using the Abenity API.',
    url='https://github.com/abenity/abenity-python',
    author='Abenity',
    license='MIT',
    cmdclass={'test': PyTest},
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Topic :: Internet :: WWW/HTTP',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.10',
    ],
    setup_requires=['setuptools>=17.1'],
    install_requires=[
        'requests>=2.31.0',
        'pycryptodome==3.20',
        'six==1.16.0'
    ],
    extras_require={'testing': ['pytest']},
    tests_require=['pytest'],
)
